#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "parse.h"
#include "y.tab.h"
#include "ex1.h"
#include "liste.h"

extern struct keyword token[MAX_TOKEN];



/*=== Add a variable into the list of variables ===*/
/*=== return : 0 : OK                           ===*/
/*===          1 : ALREADY EXISTS               ===*/
/*===          2 : ERROR                        ===*/
/*===          3 : ERROR  NAME IS NULL OR ""    ===*/
int addVariable(char * name, NodeVariable * pListeVariable)
{
  char sTmpName[2 + 1] = "";
  int  iCr = 0;

  if (name != NULL)
  {
    if (!strcmp(name, "")) return 3;

    strncpy(sTmpName, name, 2);
    sTmpName[2] = '\0';

    if (strlen(name) > 2 ) { printf("liste.c : addVariable(): La variable '%s' sera renommee en '%s'\n", name, sTmpName); }

    iCr = seekVariable(sTmpName, *pListeVariable);
    if (iCr == 0) *pListeVariable = createVariable(sTmpName, *pListeVariable);

    return iCr;
   }

  return 3;
}

/**
 * Create a new variable into the list of variables
*/
NodeVariable createVariable(char * sName, NodeVariable pBegin)
{
  NodeVariable pNew = NULL;
  NodeVariable p    = NULL;
  NodeVariable q    = NULL;

  pNew = (NodeVariable) malloc(sizeof(struct nodevariable));

  if (!pNew)  { printf("liste.c : createVariable(): Error alloc memory\n"); exit(2); }

  pNew->sName = (char *) strdup(sName);

  if (pBegin == NULL) {
        pBegin       = pNew;
        pBegin->Next = NULL;
  }
  else if (strcmp(sName, pBegin->sName) <= 0)
  {
  	pNew->Next = pBegin;
  	pBegin     = pNew;
  }
  else
  {
  	p = pBegin;
  	q = pBegin->Next;
  	while ( (q != NULL) && (strcmp(sName, q->sName) > 0) )
  	{
  	  p = q;
  	  q = q->Next;
  	}
  	p->Next    = pNew;
  	pNew->Next = q;
  }

  return pBegin;
}

/**
=== Seek a variable into the list of variables
return : 0 not found                       
        1 found                           
*/
int seekVariable(char * sName, NodeVariable pListeVariable)
{
 NodeVariable pTmp;
 int iCr;

 pTmp = pListeVariable;

 while (pTmp != NULL)
 {
   if (!strcmp(pTmp->sName, sName)) return 1;
   pTmp = pTmp->Next;
  }

 return 0;
}

/*=== Print a  list of variables ===*/
/*=== return : none              ===*/
void printVariable(NodeVariable pListeVariable)
{
 NodeVariable pTmp;
 int i = 0;

 /* printf("liste.c : printVariable(): Debut\n"); */

 pTmp = pListeVariable;
 i = 0;
 while (pTmp != NULL)
 {
   printf("i=%i, name='%s'\n", i , pTmp->sName);
   pTmp = pTmp->Next;
   i++;
  }

}

/*=== Count number variable from a list of variables ===*/
/*=== return : the number                            ===*/
int countVariable(NodeVariable pListeVariable)
{
 NodeVariable pTmp = NULL;
 int i = 0;

 pTmp = pListeVariable;
 i = 0;

 while (pTmp != NULL)
 {
   pTmp = pTmp->Next;
   i++;
  }
 return i;
}

/*=== Seek the position of variable from a list of variables ===*/
/*=== return : the position                                  ===*/
int posVariable(char * sName, NodeVariable pListeVariable)
{
 NodeVariable pTmp = NULL;
 int i = 0;

 pTmp = pListeVariable;
 i = 0;

 /* printf("liste.c : posVariable(), <%s>\n", sName); */

 while ( (pTmp != NULL) && (strcmp(sName, pTmp->sName) != 0) )
 {
   pTmp = pTmp->Next;
   i++;
  }
 return i;
}



/*=== CONSTANT INTEGER ===*/
nodeType * iConst(int value) {
 nodeType * p;

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : iConst() : out of memory");

 p->type         = tiConst;
 p->con.u.iValue = value;
 p->con.poids    = cPoidsConst;

 return p;
}


/*=== CONSTANT FLOAT ===*/
nodeType * fConst(float value) {
 nodeType * p;

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : fConst() : out of memory");

 p->type         = tfConst;
 p->con.u.fValue = value;
 p->con.poids    = cPoidsConst;

 return p;
}

/*=== CONSTANT HEXADECIMAL ===*/
nodeType * hConst(char * sValue) {
 nodeType * p;
 int i;

 if ((p = malloc(sizeof(HexaNodeType))) == NULL) yyerror("liste.c : hConst() : out of memory");

 p->type      = tHexa;
 p->con.poids = cPoidsConst;
 sscanf(sValue, "#%x", &(p->hex).iValue); 
 printf("hConst: sValue=%s, iValue=%d\n", sValue, p->hex.iValue);

 return p;
}

/*=== CONSTANT STRING ===*/
nodeType * sConst(char * value) {
 nodeType * p;


 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : sConst() : out of memory");

 p->type         = tsConst;
 p->con.u.sValue = (char *) strdup(value);
 p->con.poids    = cPoidsConst;

 return p;
}

/*===========================================================================*/


/*=== VARIABLE FLOAT TEMPORAIRE ===*/
/* Utilisation : durant la compilation */
nodeType * fVar_TEMP(char * name) {
 nodeType * p;

 /* printf("liste.c : fVar_TEMP(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : fVar() : out of memory");

 p->type = tfVar;
 strncpy(p->var.sValue, name, 3); p->var.sValue[3] = '\0';
 p->var.poids  = cPoidsVar;

 return p;
}

/*=== VARIABLE FLOAT ===*/
nodeType * fVar(char * name) {
 nodeType * p;

 /* printf("liste.c : fVar(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : fVar() : out of memory");

 p->type = tfVar;
 strncpy(p->var.sValue, name, 3);
 p->var.sValue[3]= '\0';
 p->var.poids    = cPoidsVar;

 /* Ajout du nom dans la liste des variable de type float */
 addVariable(name, &pListeVarFlt);


 return p;
}


/*=== VARIABLE INTEGER ===*/
nodeType * iVar(char * name) {
 nodeType * p;

 /* printf("liste.c : iVar(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : iVar() : out of memory");

 p->type = tiVar;
 strncpy(p->var.sValue, name, 2);
 p->var.sValue[2]= '\0';
 p->var.poids    = cPoidsVar;

 /* Ajout du nom dans la liste des variable de type integer */
 addVariable(name, &pListeVarInt);

 return p;
}


/*=== VARIABLE STRING ===*/
nodeType * sVar(char * name) {
 nodeType * p;

 /* printf("liste.c : sVar(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : sVar() : out of memory");

 p->type = tsVar;
 strncpy(p->var.sValue, name, 3);
 p->var.sValue[3]= '\0';
 p->var.poids    = cPoidsVar;

 /* Ajout du nom dans la liste des variable de type string */
 addVariable(name, &pListeVarStr);

 return p;
}

/*===========================================================================*/
/*===========================================================================*/

/**
* === ARRAY OF FLOAT ===
*/
nodeType * fArray(char * name) {
 nodeType * p;

 /* printf("liste.c : fArray(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : fArray() : out of memory");

 p->type = tfVar;
 strncpy(p->var.sValue, name, 3);
 p->var.sValue[3]= '\0';
 p->var.poids    = cPoidsArray;

 /* Ajout du nom dans la liste des variable de type arrray of float */
 addVariable(name, &pListeArrayFlt);

 return p;
}


/**
* === ARRAY OF INTEGER ===
*/
nodeType * iArray(char * name) {
 nodeType * p;

 /* printf("liste.c : iArray(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : iArray() : out of memory");

 p->type = tiArray;
 strncpy(p->var.sValue, name, 2);
 p->var.sValue[2]= '\0';
 p->var.poids    = cPoidsArray; 

 /* Ajout du nom dans la liste des variable de type array of integer */
 addVariable(name, &pListeArrayInt);

 return p;
}


/**
* === ARRAY OF STRING ===
*/
nodeType * sArray(char * name) {
 nodeType * p;

 /* printf("liste.c : sArray(): Debut\n"); */

 if ((p = malloc(sizeof(ConstNodeType))) == NULL) yyerror("liste.c : sArray() : out of memory");

 p->type = tsArray;
 strncpy(p->var.sValue, name, 3);
 p->var.sValue[3]= '\0';
 p->var.poids    = cPoidsArray;

 /* Ajout du nom dans la liste des variable de type array of string */
 addVariable(name, &pListeArrayStr);

 return p;
}

/*=== Add a line into the list of line          ===*/
/*=== return : 0 : OK                           ===*/
/*===          1 : Error                        ===*/
int addLine(nodeType * sLine, struct nodetypeliste * * pListeLine)
{
  nodeTypeListe * pTmpType = NULL;

  pTmpType = (nodeTypeListe *) malloc(sizeof(nodeTypeListe));

  if (pTmpType == NULL) { printf("liste.c : addLine() : pTmpType NULL\n"); return 1; }

  pTmpType->Node = sLine;

  if (*pListeLine == NULL) {
         pTmpType->Next = NULL;
         pTmpType->Prev = NULL;
        *pListeLine     = pTmpType;
         pFIRSTLINEPRG  = pTmpType; /* PERMET DE RECUPERER LA 1ERE LIGNE DU PRG */
  } else {
        pTmpType->Next      = (*pListeLine)->Next;
        (*pListeLine)->Next = pTmpType;
        pTmpType->Prev      = *pListeLine;
        *pListeLine         = pTmpType;
  }

  return 0;
}

/*===========================================================================*/
/*===========================================================================*/


/*=== OPERATION ===*/
nodeType * Op(int oper, int nops, ...) {
 nodeType * p = NULL;
 va_list    ap;
 size_t     size;
 int        i = 0;

 printf("liste.c : op() : BEGIN\n");
 printf("liste.c : op() : DEBUG : oper=<%i>, nops=<%i>\n", oper, nops);
 
 size = sizeof(OpNodeType) + (nops - 1) * sizeof(nodeType * );

 if ((p = malloc(size)) == NULL) yyerror("liste.c : op() : out of memory\n");

 printf("liste.c : op() : SUITE\n");
 
 p->type     = tOp;
 p->op.oper  = oper;
 p->op.nops  = nops;
 p->op.poids = 1; //cPoidsOp;

 printf("liste.c : op() : SUITE1\n");
 
 // Un petit affichage ...
 if (oper == LIGNE)
    printf("liste.c : Op() : oper(num=%d)='LIGNE', nops=%d\n", oper, nops);
 else
   if (oper == vSEPARATOR)
     printf("liste.c : Op() : oper(num=%d)='%s', nops=%d\n", oper, "SEPARATEUR", nops);
   else     
      printf("liste.c : Op() : oper(num=%d)='%s', nops=%d\n", oper, oper < 1000 ? token[oper].name : "", nops);

 printf("liste.c : op() : SUITE2\n");
 
 va_start(ap, nops);
 for (i = 0; i < nops; i++) {
     printf("liste.c : op() : BOUCLE\n");
     p->op.args[i] = va_arg(ap, nodeType *);
     if (p->op.args[i] != NULL) {
             printf("liste.c : Op() : Op(%i)='%s'", i, tabTYPE_OP[p->op.args[i]->op.type]);
             if (p->op.args[i]->op.type == tiConst) printf(", value='%d'\n", p->op.args[i]->con.u.iValue);
        else if (p->op.args[i]->op.type == tsConst) printf(", value='%s'\n", p->op.args[i]->con.u.sValue);
        else printf("\n");
     }
 }
 
 va_end(ap);
 
 printf("liste.c : op() : FIN\n");
 
 return p;
}

/*=== LIBERATION DE LA MEMOIRE ALLOUEE ===*/
void freeNode(nodeType * p) {
 int i;

 if (!p) return;
 if (p->type == tOp) for (i = 0; i < p->op.nops; i++) freeNode(p->op.args[i]);
 free(p);
}


/*=== AFFICHAGE DE LA CONSTITUTION D'UN NOEUD ===*/
/* 
*/
int print(nodeType * Node) {
 int i;

 printf("liste.c : print() : DEBUT\n");
 
 if (!Node) { printf("liste.c : print(): Node is null. Sortie\n"); return 1; }

 /*
 if (Node->type == tOp)
 {
  if (Node->op.oper == LIGNE) {
     if (Node->op.args[1] != NULL) print(Node->op.args[1]);
  } else {
     if (Node->op.args[0] != NULL) print(Node->op.args[0]);
     if (Node->op.args[1] != NULL) print(Node->op.args[1]);
  }
 }
 */
 printf("liste.c : print() : \n --- liste.c : print(): type=%d, oper=%d\n", Node->type, Node->op.oper);

 switch (Node->type) {
        case tOp     :
                       /* Traitement special dans le cas d'une ligne */
                       if (Node->op.oper == LIGNE) {
                          fprintf(fPrg, "L_%d: \n", Node->op.args[0]->con.u.iValue);
                          printf("liste.c : print() : type=OP, oper=%d, LIGNE, L_%d\n", Node->op.oper, Node->op.args[0]->con.u.iValue);
                          print(Node->op.args[1]);
                       } else {
                           if (Node->op.oper < 1000) {
                              /* printf("debug : liste.c : print(): fct(), oper=%d AVANT \n", Node->op.oper); */
                              /* token[Node->op.oper].fct(Node); */
                              printf("liste.c : print() : liste.c : print() : '%s, oper=%i'\n", token[Node->op.oper].name, Node->op.oper);
                              token[Node->op.oper].fct(Node);
			      printf("\n");
                              /* printf("debug : liste.c : print(): fct(), oper=%d APRES\n", Node->op.oper); */
                           } else {
			      // Si c'est un séparateur
			      if (Node->op.oper == 4000) {
				print(Node->op.args[0]);
				print(Node->op.args[1]);
			      } else {
				printf("liste.c : print() : |token ='%d'|", Node->op.oper);
			      }
                           }
                       }
                       break;
        case tfConst : printf("liste.c : print() : fCst=%f\n", Node->con.u.fValue);     break;
        case tiConst : printf("liste.c : print() : iCst=%d\n", Node->con.u.iValue);     break;
        case tsConst : printf("liste.c : print() : sCst=%s\n", Node->con.u.sValue);     break;
        case tiVar   : printf("liste.c : print() : iVar=%s\n", Node->var.sValue);       break;
        case tfVar   : printf("liste.c : print() : fVar=%s\n", Node->var.sValue);       break;
        case tsVar   : printf("liste.c : print() : sVar=%s\n", Node->var.sValue);       break;
        case tHexa   : printf("liste.c : print() : hexa=%d\n", Node->hex.iValue);       break;
        default      : printf("\nliste.c : print() : Erreur\n");                        break;
 }
 printf("liste.c : print() : FIN\n");
}

/*
* === AFFICHAGE DE LA CONSTITUTION D'UN NOEUD ===
*/
void printAFF (nodeType * Node, int tab) {
 int i;
 char atab[10];
 
/* printf("liste.c : printAFF() : DEBUT\n"); */
 
 if (!Node) { printf("liste.c : printAFF() : Node is null. Sortie\n"); return ;}

 /* Calcul affichage du poids */
 /* printf("liste.c : printAFF() : poids=%i\n", getPoids(Node)); */
 
 switch (Node->type) {
        case tOp     :
	               /* printf("liste.c : printAFF() : switch : op\n"); */
                       /* Traitement special dans le cas d'une ligne */
                       if (Node->op.oper == LIGNE) {
                          printf("\nliste.c : printAFF(): LIGNE\n");
                       } else {
			   /* printf("liste.c : printAFF() : switch : op : autre\n"); */
                           if (Node->op.oper < 1000) {
			      /* printf("liste.c : printAFF() : switch : op : autre < 1000, oper=<%i>\n", Node->op.oper); */
                              printf("liste.c : printAFF() : '%s'\n", token[Node->op.oper].name);
                           } else {
			      /* printf("liste.c : printAFF() : switch : autre > 1000\n"); */
                              printf("liste.c : printAFF() : |token ='%d'|\n", Node->op.oper);
                           }
                        }
                       break;
        case tfConst : printf("liste.c : printAFF() : fCst=%f\n", Node->con.u.fValue); break;
        case tiConst : printf("liste.c : printAFF() : iCst=%d\n", Node->con.u.iValue); break;
        case tsConst : printf("liste.c : printAFF() : sCst=%s\n", Node->con.u.sValue); break;
        case tiVar   : printf("liste.c : printAFF() : iVar=%s\n", Node->var.sValue  ); break;
        case tfVar   : printf("liste.c : printAFF() : fVar=%s\n", Node->var.sValue  ); break;
        case tsVar   : printf("liste.c : printAFF() : sVar=%s\n", Node->var.sValue  ); break;
        case tHexa   : printf("liste.c : printAFF() : hexa=%d\n", Node->hex.iValue  ); break;
        default      : printf("\nliste.c : printAFF() : Erreur\n");                    break;
 }

/* printf("liste.c : printAFF() : Si opération\n");*/

 if (Node->type == tOp)
 {
  if (Node->op.oper == LIGNE) {
     printf(" fg : "); if (Node->op.args[0] != NULL) printAFF(Node->op.args[0], tab+1); else printf("\n");
     printf(" fd : "); if (Node->op.args[1] != NULL) printAFF(Node->op.args[1], tab+1); else printf("\n");
  } else {
    if (tab == '0') {
      atab[0] = '\t'; atab[1] = '\0';
    }
    else
    {
      int i = 0;
      for (i ; i < tab; i++) { atab[i] = '\t'; }
      atab[tab] = '\0';
    }
    for (i=0;i < Node->op.nops;i++) {
       /* printf("liste.c : printAFF() : boucle, i=<%i>\n", i);*/
       if (Node->op.args[i] != NULL) { printf("%s- %i : ", atab, i); printAFF(Node->op.args[i], tab+1); };
    }
  }
 }
 // La fin
 /* printf("liste.c : printAFF() : FIN\n"); */
}

/*=== AFFICHAGE DE LA CONSTITUTION D'UN NOEUD ===*/
void printNODE (nodeType * Node) {

 if (Node->type == tOp)
 { 
   if (Node->op.args[0] != NULL) printNODE(Node->op.args[0]);
   if (Node->op.args[1] != NULL) printNODE(Node->op.args[1]);
 }

 if (!Node) { printf("liste.c : printNODE() : Node is null. Sortie\n"); return ;}

 switch (Node->type) {
        case tOp     : printf("%s ", token[Node->op.oper].name);    break;
        case tfConst : printf("%f ", Node->con.u.fValue);           break;
        case tiConst : printf("%d ", Node->con.u.iValue);           break;
        case tsConst : printf("%s ", Node->con.u.sValue);           break;
        case tiVar   : printf("%s ", Node->var.sValue  );           break;
        case tfVar   : printf("%s ", Node->var.sValue  );           break;
        case tsVar   : printf("%s ", Node->var.sValue  );           break;
        case tHexa   : printf("%d ", Node->hex.iValue  );           break;
        default      : printf("\nliste.c : printNODE() : ERREUR\n"); break;
 }
}

/*--------------------------------------------------------------------------------*/
/*
*
*/
int CalculPoids(nodeType *Node) {
  int iPoidsGauche = 0;
  int iPoidsDroit = 0;
  switch (Node->type) {
   case tOp     : 
                 iPoidsGauche = getPoids(Node->op.args[0]);
                 iPoidsDroit  = getPoids(Node->op.args[1]);
		 // Cas des fonctions unaires (sin, log, ...)
		 if ((iPoidsDroit == 0) &&  (iPoidsGauche == 1)) return 2;
		 return Node->op.poids; 
		 break;
   case tfConst : return Node->con.poids; break;
   case tiConst : return Node->con.poids; break;
   case tsConst : return Node->con.poids; break;
   case tiVar   : return Node->var.poids; break;
   case tfVar   : return Node->var.poids; break;
   case tsVar   : return Node->var.poids; break;
   default      : printf("\nliste.c : CalculPoids() : ERREUR\n");  break;
  }
 
  /* Erreur */
   return -1;
}

/*--------------------------------------------------------------------------------*/
/*
* Permet de déterminer le poids d'un noeud.
*/ 
int getPoids(nodeType * Node) {
 int iPoidsGauche;
 int iPoidsDroit;

 if (Node == NULL) return 0;

 switch (Node->type) {
  case tOp     : iPoidsGauche = getPoids(Node->op.args[0]);
                 iPoidsDroit  = getPoids(Node->op.args[1]);
		 // Cas des fonctions unaires (sin, log, ...)
		 if (iPoidsDroit == 0) {
		   if (iPoidsGauche == 1) return iPoidsGauche + 1;
		   return iPoidsGauche;
		 } else {
		      if (iPoidsGauche > iPoidsDroit) return iPoidsGauche;
		      if (iPoidsGauche < iPoidsDroit) return iPoidsDroit;
		      return iPoidsGauche+1;
		 }
                 break;
  case tfConst : return 1; break;
  case tiConst : return 1; break;
  case tsConst : return 1; break;
  case tiVar   : return 1; break;
  case tfVar   : return 1; break;
  case tsVar   : return 1; break;
  default      : printf("\nliste.c : getPoids() : Erreur\n");  break;
 }
}

/*--------------------------------------------------------------------------------*/
/*
* Permet de recalculer le poids de chaque noeud
*/ 
int recalculePoids(nodeType * Node) {
 int iPoidsGauche;
 int iPoidsDroit;

 printf("liste.c : recalculePoids() : DEBUT\n");
 
 if (Node == NULL) return 0;

 /* printf("liste.c : recalculePoids() : LE TRAITEMENT\n");
 printf("liste.c : recalculePoids() : le pointeur=%p\n", Node);
 */
 
 switch (Node->type) {
  case tOp     : printf("liste.c : recalculePoids() : une operation \n");
                 iPoidsGauche = recalculePoids(Node->op.args[0]);
                 iPoidsDroit  = recalculePoids(Node->op.args[1]);
		 if (iPoidsGauche > iPoidsDroit) { Node->op.poids=iPoidsGauche ; return iPoidsGauche; }
		 if (iPoidsGauche < iPoidsDroit) { Node->op.poids=iPoidsDroit  ; return iPoidsDroit; }
		 Node->op.poids=iPoidsGauche + 1;
		 return iPoidsGauche+1;		 
                 break;
  case tfConst : Node->con.poids=1; return 1; break;
  case tiConst : Node->con.poids=1; return 1; break;
  case tsConst : Node->con.poids=1; return 1; break;
  case tiVar   : Node->var.poids=1; return 1; break;
  case tfVar   : Node->var.poids=1; return 1; break;
  case tsVar   : Node->var.poids=1; return 1; break;
  default      : printf("\nliste.c : recalculePoids() : Erreur\n");  break;
 }
 printf("liste.c : recalculePoids() : FIN\n");
}

/*=========================*/
/*=== PRINT THE PROGRAM ===*/
/*=========================*/
int printPRG(struct nodetypeliste * pListeLine) {
 struct nodetypeliste * pTmpListLine = NULL;

 if (!pListeLine) { printf("liste.c : printPRG() : pListeLine is null. Sortie\n"); return ;}
 
 pTmpListLine = pListeLine;
 
 while (pTmpListLine != NULL)
 {  
  printAFF(pTmpListLine->Node, 0);
  pTmpListLine = pTmpListLine->Next;
 }
 printf("liste.c : printPRG() : fin \n");
}

/*==================================================================*/
/*==================================================================*/
nodeType * Arbre_pas_trop_grand_maximal(nodeType * Node) {
 int iPoids = 0;
 int iPoidsGauche = 0;
 int iPoidsDroit = 0;
 
 /* Info */
 printf("liste.c : Arbre_pas_trop_grand_maximal() : DEBUT : poids=%p, poids.gauche=%p, poids.droit=%p\n", Node, Node->op.args[0], Node->op.args[1]);
 
 /* Le traitement */
 switch (Node->type) {
  case tOp     : iPoids = getPoids(Node); 
                 iPoidsGauche = getPoids(Node->op.args[0]);
		 iPoidsDroit  = getPoids(Node->op.args[1]);
		 printf("liste.c : Arbre_pas_trop_grand_maximal() : op : poids=%p, poids.gauche=%p, poids.droit=%p\n", Node, Node->op.args[0], Node->op.args[1]);
                 break;
  case tfConst : 
  case tiConst : 
  case tsConst : iPoids = Node->con.poids; break;
  case tiVar   : 
  case tfVar   : 
  case tsVar   : iPoids = Node->var.poids; break;
  default      : printf("\nliste.c : Arbre_pas_trop_grand_maximal() : Erreur\n");  break;
 }
 
 printf("liste.c : Arbre_pas_trop_grand_maximal() : poids=%d, poids.gauche=%d, poids.droit=%d\n", iPoids, iPoidsGauche, iPoidsDroit);
 
 if (iPoids <= 2) 
    return Node;
 else
    if (iPoidsGauche > 2) {
       printf("liste.c : Arbre_pas_trop_grand_maximal() : le gauche, pointeur=%p\n", Node->op.args[0]);
       return Arbre_pas_trop_grand_maximal(Node->op.args[0]);
    }
    else if (iPoidsDroit >= 2) {
       printf("liste.c : Arbre_pas_trop_grand_maximal() : le droit, pointeur=%p\n", Node->op.args[1]);
       return Arbre_pas_trop_grand_maximal(Node->op.args[1]);    
    }
}

  
/*
* Génération du code
*/
void generate(nodeType * Node) {
  int iT = 1;
  char buffer[4] = "\n";
  nodeType * Node_reduit = NULL;
  
  while (Node != NULL)  {
    printf("liste.c : generate() : recalculePoids(), Node=%p\n", Node);
    int i = recalculePoids(Node);
    
    /* L'arbre le plus petit compilable */
    printf("liste.c : generate() : pointeur Node=%p, Node_reduit=%p\n", Node, Node_reduit);
    printf("liste.c : generate() : Arbre_pas_trop_grand_maximal()\n");    
    Node_reduit = Arbre_pas_trop_grand_maximal(Node);
    printf("liste.c : generate() : pointeur Node_reduit=%p\n", Node_reduit);
    
    /* Generation du code */
    printf("liste.c : generate() : eval()\n");
    (void)eval(Node_reduit, 1);
    
    /* Stockage du résultat dans une variable temporaire si necessaire */
    if (Node_reduit != (Node)) {
      printf("liste.c : generate() : stockage dans variable temporaire\n");
      
      /* Le code pour sauvegarder dans un registre temporaire */
      fprintf(fPrg, "\tlda v0ACCU1\n");   fprintf(fPrg, "\tsta T%d\n"   , iT);
      fprintf(fPrg, "\tlda v0ACCU1+1\n"); fprintf(fPrg, "\tsta T%d+1\n" , iT);
      fprintf(fPrg, "\tlda v0ACCU1+2\n"); fprintf(fPrg, "\tsta T%d+2\n" , iT);
      fprintf(fPrg, "\tlda v0ACCU1+3\n"); fprintf(fPrg, "\tsta T%d+3\n" , iT);
      fprintf(fPrg, "\tlda v0ACCU1+4\n"); fprintf(fPrg, "\tsta T%d+4\n" , iT);
      
      /* Node_reduit = variable temporaire  */
      sprintf(buffer, "T%d", iT);      
      
      *Node_reduit = *fVar_TEMP(buffer);    
      
      printf("liste.c : generate() : pointeur Node_reduit=%p, iT=%d\n", Node_reduit, iT);
      
      /* Incrémentation de la variable temporaire */
      iT = iT + 1;
      
      /* A supprimer */
      int j = 0;
      printf("attente="); scanf("%i", &j);
    } else {
      Node = NULL;
    }      
  }  
}
  
/*============================*/
/*=== GENERATE THE PROGRAM ===*/
/*============================*/
int printPRG_generate(struct nodetypeliste * pListeLine) {
 struct nodetypeliste * pTmpListLine = NULL;

 printf("\nliste.c : printPRG_generate() : DEBUT\n\n");
  
 if (!pListeLine) { printf("liste.c : printPRG_generate() : pListeLine is null. Sortie\n"); return ;}
 
 pTmpListLine = pListeLine;
 
 while (pTmpListLine != NULL)
 {
  generate(pTmpListLine->Node);
  pTmpListLine = pTmpListLine->Next;
 }
 printf("\nliste.c : printPRG_generate() : FIN\n");
}
