/*=== Variable global ===*/
#include <stdio.h>

int iLib1;
int iLib2;

#define cPoidsConst ((int) 1)
#define cPoidsVar   ((int) 2)
#define cPoidsArray ((int) 2)
#define cPoidsOp    ((int) 3)


/*=== TYPEDEF ===*/

struct nodevariable {
 char * sName;
 struct nodevariable * Next;
};
typedef struct nodevariable * NodeVariable;

typedef enum { tfConst, tiConst, tsConst, tHexa, tfVar, tiVar, tsVar, tfArray, tiArray, tsArray, tOp } nodeEnum;

typedef struct {
	nodeEnum type;
	union typeValue { int iValue; float fValue; char * sValue; } u;
	int      poids; // le poids
} ConstNodeType;

typedef struct {
	nodeEnum type;
	int      iValue;
	int      poids; // le poids
} HexaNodeType;


typedef struct {
	nodeEnum type;
	char sValue[3 + 1];
	int      poids; // le poids
} VarNodeType;

typedef struct {
	nodeEnum type;  // type de noeud
	int      oper;  // l'operation
	int      nops;  // Nombre d'operation
	int      poids; // le poids
	union nodeTypeTag * args[1];
} OpNodeType;

typedef union nodeTypeTag {
	nodeEnum      type;	
	ConstNodeType con;
	HexaNodeType  hex;
	VarNodeType   var;
	OpNodeType    op;
} nodeType;	

 
typedef struct nodetypeliste {
	nodeType             * Node;
	struct nodetypeliste * Next;
	struct nodetypeliste * Prev;	
} nodeTypeListe; 

struct info_prg {
	int iNbFOR;              /* Nombre de mot-cle FOR    */
	int iNbNEXT;             /* Nombre de mot-cle NEXT   */
	int iNbREPEAT;           /* Nombre de mot-cle REPEAT */
	int iNbUNTIL;            /* Nombre de mot-cle UNTIL  */
	int iNbPOP;              /* Nombre de mot-cle POP    */
	int iNbPULL;             /* Nombre de mot-cle PULL   */
};
typedef struct info_prg INFO_PRG;

/*=== VARIABLES GLOBALES ===*/
/* nodeType * pPRG; */
struct nodetypeliste * pPRG;
struct nodetypeliste * pFIRSTLINEPRG;
/*nodeType * pListeLines;*/
struct nodetypeliste * pListeLines;

/* DEFINITION DES LISTE DE VARIABLES SIMPLES */
NodeVariable  pListeVarInt;
NodeVariable  pListeVarFlt;
NodeVariable  pListeVarStr;

/* DEFINITION DES LISTE DE VARIABLES DEFINIES PAR DIM */
NodeVariable  pListeArrayInt;
NodeVariable  pListeArrayFlt;
NodeVariable  pListeArrayStr;

int           iNumString;

FILE          * fHeader;
FILE          * fCstStr;
FILE          * fPrg;

/*=== PROTOTYPES ===*/
int          addVariable   (char * name , NodeVariable * pListeVariable);
NodeVariable createVariable(char * sName, NodeVariable   pBegin);
int          seekVariable  (char * sName, NodeVariable   pListeVariable);
void         printVariable (NodeVariable pListeVariable);
int          countVariable (NodeVariable pListeVariable);
int          posVariable   (char * sName, NodeVariable pListeVariable);

nodeType * iConst (int     value);
nodeType * fConst (float   value);
nodeType * hConst (char  * value);
nodeType * sConst (char  * value);
nodeType * iVar   (char  * name);
nodeType * fVar   (char  * name);
nodeType * sVar   (char  * name);
nodeType * iArray (char  * name);
nodeType * fArray (char  * name);
nodeType * sArray (char  * name);

nodeType * Op     (int oper, int nops, ...);
int        print  (nodeType * Node);
void      freeNode(nodeType * p);

int printPRG(struct nodetypeliste * pListeLine);
int addLine(nodeType * sLine, struct nodetypeliste * * pListeLine);

/*=== CONSTANTES ===*/
#define PLUS                  ((int)    1)
#define MINUS                 ((int)    2)
#define DIVIDE                ((int)    3)
#define MULTIPLY              ((int)    4)
#define POWER                 ((int)    5)
#define EQ                    ((int)    6)
#define NE                    ((int)    7)
#define LT                    ((int)    8)
#define LE                    ((int)    9)
#define GT                    ((int)   10)
#define GE                    ((int)   11)
#define AT                    ((int)   12)

#define K7_SLOW               ((int)   20)
#define K7_AUTO               ((int)   21)
#define K7_BEGIN              ((int)   22)
#define K7_END                ((int)   23)
#define K7_JOIN               ((int)   24)
#define K7_VERIFY             ((int)   25)

#define LISTE                 ((int) 1000)
#define ARG                   ((int) 1010)
#define PROGRAMME             ((int) 1100)
#define LIGNE                 ((int) 1200)
#define NOP                   ((int) 2000)
#define ARRAY                 ((int) 3000)
#define SEPARATEUR            ((int) 4000)


#define ABS                   ((int)  216)
#define AND                   ((int)  209)
#define ASC                   ((int)  236)
#define ATN                   ((int)  229)
#define AUTO                  ((int)  199)
#define CALL                  ((int)  191)
#define CHAR                  ((int)  176)
#define CHR                   ((int)  237)
#define CIRCLE                ((int)  173)
#define CLEAR                 ((int)  189)
#define CLOAD                 ((int)  182)
#define CLS                   ((int)  148)
#define CONT                  ((int)  187)
#define COS                   ((int)  226)
#define CSAVE                 ((int)  183)
#define CURMOV                ((int)  171)
#define CURSET                ((int)  170)
#define DATA                  ((int)  145)
#define DEEK                  ((int)  231)
#define DEF                   ((int)  184)
#define DIM                   ((int)  147)
#define DOKE                  ((int)  138)
#define DRAW                  ((int)  172)
#define EDIT                  ((int)  129)
#define ELSE                  ((int)  200)
#define END                   ((int)  128)
#define EXP                   ((int)  225)
#define EXPLODE               ((int)  164)
#define FALSE                 ((int)  240)
#define FILL                  ((int)  175)
#define FN                    ((int)  196)
#define FOR                   ((int)  141)
#define FRE                   ((int)  218)
#define GET                   ((int)  190)
#define GO                    ((int)  247)
#define GOSUB                 ((int)  155)
#define GOTO                  ((int)  151)
#define GRAB                  ((int)  159)
#define HEX                   ((int)  220)
#define HIMEM                 ((int)  158)
#define HIRES                 ((int)  162)
#define IF                    ((int)  153)
#define INK                   ((int)  178)
#define INPUT                 ((int)  146)
#define INT                   ((int)  215)
#define KEY                   ((int)  241)
#define LEFT                  ((int)  244)
#define LEN                   ((int)  233)
#define LET                   ((int)  150)
#define LIST                  ((int)  188)
#define LLIST                 ((int)  142)
#define LN                    ((int)  224)
#define LOG                   ((int)  232)
#define LORES                 ((int)  137)
#define LPRINT                ((int)  143)
#define MID                   ((int)  146)
#define MUSIC                 ((int)  168)
#define NEW                   ((int)  193)
#define NEXT                  ((int)  144)
#define NOT                   ((int)  202)
#define ON                    ((int)  180)
#define OR                    ((int)  210)
#define PAPER                 ((int)  177)
#define PATTERN               ((int)  174)
#define PEEK                  ((int)  230)
#define PI                    ((int)  238)
#define PING                  ((int)  166)
#define PLAY                  ((int)  169)
#define PLOT                  ((int)  135)
#define POINT                 ((int)  243)
#define POKE                  ((int)  185)
#define POP                   ((int)  134)
#define POS                   ((int)  219)
#define PRINT                 ((int)  186)
#define PULL                  ((int)  136)
#define READ                  ((int)  149)
#define RECALL                ((int)  131)
#define RELEASE               ((int)  160)
#define REM                   ((int)  157)
#define REPEAT                ((int)  139)
#define RESTORE               ((int)  154)
#define RETURN                ((int)  156)
#define RIGHT                 ((int)  245)
#define RND                   ((int)  223)
#define RUN                   ((int)  152)
#define SCRN                  ((int)  242)
#define SGN                   ((int)  214)
#define SHOOT                 ((int)  163)
#define SIN                   ((int)  227)
#define SOUND                 ((int)  167)
#define SPC                   ((int)  197)
#define SQR                   ((int)  222)
#define STEP                  ((int)  283)
#define STOP                  ((int)  179)
#define STORE                 ((int)  130)
#define STR                   ((int)  234)
#define TAB                   ((int)  194)
#define TAN                   ((int)  228)
#define TEXT                  ((int)  161)
#define THEN                  ((int)  201)
#define TO                    ((int)  195)
#define TROFF                 ((int)  133)
#define TRON                  ((int)  132)
#define TRUE                  ((int)  239)
#define UNTIL                 ((int)  140)
#define USR                   ((int)  217)
#define VAL                   ((int)  235)
#define WAIT                  ((int)  181)
#define ZAP                   ((int)  165)
#define AMPERSAND             ((int)  100)
#define SHRIEK                ((int)  101)

